@@ -1,11 +1,12 @@
-/* $Id: ban.cpp 38304 2009-09-02 08:22:27Z soliton $ */
+/* $Id: ban.cpp 52533 2012-01-07 02:35:17Z shadowmaster $ */
 /*
-   Copyright (C) 2008 - 2009 by Pauli Nieminen <paniemin@cc.hut.fi>
+   Copyright (C) 2008 - 2012 by Pauli Nieminen <paniemin@cc.hut.fi>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -13,10 +14,13 @@
 */
 
 #include "config.hpp"
+#include "foreach.hpp"
 #include "log.hpp"
 #include "filesystem.hpp"
 #include "serialization/parser.hpp"
 #include "serialization/binary_or_text.hpp"
+#include "serialization/string_utils.hpp"
+#include "util.hpp"
 
 #include "ban.hpp"
 
@@ -25,9 +29,11 @@
 
 namespace wesnothd {
 
-#define ERR_SERVER LOG_STREAM(err, mp_server)
-#define LOG_SERVER LOG_STREAM(info, mp_server)
-#define DBG_SERVER LOG_STREAM(debug, mp_server)
+
+static lg::log_domain log_server("server");
+#define ERR_SERVER LOG_STREAM(err, log_server)
+#define LOG_SERVER LOG_STREAM(info, log_server)
+#define DBG_SERVER LOG_STREAM(debug, log_server)
 
 	std::ostream& operator<<(std::ostream& o, const banned& n)
 	{
@@ -46,20 +52,6 @@ namespace wesnothd {
 
 	banned_compare_subnet::compare_fn banned_compare_subnet::active_ = &banned_compare_subnet::less;
 
-	void banned_compare_subnet::set_use_subnet_mask(bool use)
-	{
-		if (use)
-		{
-			assert(active_ == &banned_compare_subnet::less);
-			active_ = &banned_compare_subnet::less_with_subnet;
-		}
-		else
-		{
-			assert(active_ != &banned_compare_subnet::less);
-			active_ = &banned_compare_subnet::less;
-		}
-	}
-
 	bool banned_compare_subnet::operator()(const banned_ptr& a, const banned_ptr& b) const
 	{
 		return (this->*(active_))(a,b);
@@ -70,20 +62,6 @@ namespace wesnothd {
 		return a->get_int_ip() < b->get_int_ip();
 	}
 
-	bool banned_compare_subnet::less_with_subnet(const banned_ptr& a, const banned_ptr& b) const
-	{
-		return a->get_mask_ip(b->mask()) < b->get_mask_ip(a->mask());
-	}
-
-	subnet_compare_setter::subnet_compare_setter()
-	{
-		banned_compare_subnet::set_use_subnet_mask(true);
-	}
-
-	subnet_compare_setter::~subnet_compare_setter()
-	{
-		banned_compare_subnet::set_use_subnet_mask(false);
-	}
 	const std::string banned::who_banned_default_ = "system";
 
 	banned_ptr banned::create_dummy(const std::string& ip)
@@ -191,23 +169,23 @@ namespace wesnothd {
 	{
 		{
 			// parse ip and mask
-			ip_text_ = cfg["ip"];
+			ip_text_ = cfg["ip"].str();
 			ip_mask pair = parse_ip(ip_text_);
 			ip_ = pair.first;
 			mask_ = pair.second;
 		}
-		nick_ = cfg["nick"];
+		nick_ = cfg["nick"].str();
 		if (cfg.has_attribute("end_time"))
 			end_time_ = lexical_cast_default<time_t>(cfg["end_time"], 0);
 		if (cfg.has_attribute("start_time"))
 			start_time_ = lexical_cast_default<time_t>(cfg["start_time"], 0);
-		reason_ = cfg["reason"];
+		reason_ = cfg["reason"].str();
 
 		// only overwrite defaults if exists
 		if (cfg.has_attribute("who_banned"))
-			who_banned_ = cfg["who_banned"];
+			who_banned_ = cfg["who_banned"].str();
 		if (cfg.has_attribute("group"))
-			group_ = cfg["group"];
+			group_ = cfg["group"].str();
 	}
 
 	void banned::write(config& cfg) const
@@ -282,12 +260,10 @@ namespace wesnothd {
 		scoped_istream ban_file = istream_file(filename_);
 		read_gz(cfg, *ban_file);
 
-		const config::child_list& bans = cfg.get_children("ban");
-		for (config::child_list::const_iterator itor = bans.begin();
-				itor != bans.end(); ++itor)
+		foreach (const config &b, cfg.child_range("ban"))
 		{
 			try {
-				banned_ptr new_ban(new banned(**itor));
+				banned_ptr new_ban(new banned(b));
 				assert(bans_.insert(new_ban).second);
 
 				if (new_ban->get_end_time() != 0)
@@ -298,15 +274,12 @@ namespace wesnothd {
 		}
 
 		// load deleted too
-		if (cfg.child("deleted"))
+		if (const config &cfg_del = cfg.child("deleted"))
 		{
-			const config& cfg_del = *cfg.child("deleted");
-			const config::child_list& del_bans = cfg_del.get_children("ban");
-			for (config::child_list::const_iterator itor = del_bans.begin();
-					itor != del_bans.end(); ++itor)
+			foreach (const config &b, cfg_del.child_range("ban"))
 			{
 				try {
-					banned_ptr new_ban(new banned(**itor));
+					banned_ptr new_ban(new banned(b));
 					deleted_bans_.push_back(new_ban);
 				} catch (banned::error& e) {
 					ERR_SERVER << e.message << " while reading deleted bans\n";
@@ -393,7 +366,7 @@ namespace wesnothd {
 		default_ban_times::const_iterator time_itor = ban_times_.find(duration);
 		if (utils::lowercase(duration) == "permanent" || duration == "0") {
 			*time = 0;
-		} else if (time_itor != ban_times_.end()) {
+		} else if (ban_times_.find(duration) != ban_times_.end()) {
 			*time += time_itor->second;
 		} else {
 			std::string::const_iterator i = duration.begin();
@@ -651,7 +624,7 @@ namespace wesnothd {
 		}
 
 		// Don't list ban groups when looking for specific bans.
-		if (!groups.empty() && mask != "*")
+		if (!groups.empty() && mask == "*")
 		{
 			out << "\nban groups: ";
 
@@ -673,7 +646,8 @@ namespace wesnothd {
 		}
 		ban_set::const_iterator ban = std::find_if(bans_.begin(), bans_.end(), boost::bind(&banned::match_ip, boost::bind(&banned_ptr::get, _1), pair));
 		if (ban == bans_.end()) return "";
-		return (*ban)->get_reason();
+		const std::string& nick = (*ban)->get_nick();
+		return (*ban)->get_reason() + (nick.empty() ? "" : " (" + nick + ")");
 	}
 
 	void ban_manager::init_ban_help()
@@ -706,20 +680,17 @@ namespace wesnothd {
 	void ban_manager::load_config(const config& cfg)
 	{
 		ban_times_.clear();
-		const config::child_list& times = cfg.get_children("ban_time");
-		for (config::child_list::const_iterator itor = times.begin();
-				itor != times.end(); ++itor)
-		{
+		foreach (const config &bt, cfg.child_range("ban_time")) {
 			time_t duration = 0;
-			if (parse_time((**itor)["time"], &duration)) {
-				ban_times_.insert(default_ban_times::value_type((**itor)["name"], duration));
+			if (parse_time(bt["time"], &duration)) {
+				ban_times_.insert(default_ban_times::value_type(bt["name"], duration));
 			}
 		}
 		init_ban_help();
 		if (filename_ != cfg["ban_save_file"])
 		{
 			dirty_ = true;
-			filename_ = cfg["ban_save_file"];
+			filename_ = cfg["ban_save_file"].str();
 		}
 	}
 
